/*
The microcontroller sends a start sequence to start
a pressure or temperature measurement. After
converting time (4.5ms), the result value (UP or UT,
respectively) can be read via the I2C interface. For
calculating temperature in C and pressure in hPa,
the calibration data has to be used. These
constants can be read out from the BMP085
EEPROM via the I2C interface at software
initialization.

set oversampling_setting to 3

start temp measurement, wait 4.5ms, read UT, start press measurement, 25.5ms (high resolution), read UP, calculate pressure/temp in physical units

UP is 16 to 19 bit pressure data
uT is 16 bit temperature data

if I have more problems, see this thread:
http://www.societyofrobots.com/robotforum/index.php?topic=12512.msg103509#msg103509
*/

// Oversampling Setting: 0,1,2,3
#define OSS 	3
#define tweak_altitude	0//a constant added to altitude, if you wanted to shift by a value

int16_t ac1;
int16_t ac2;
int16_t ac3; 
uint16_t ac4;
uint16_t ac5;
uint16_t ac6;
int16_t b1; 
int16_t b2;
int16_t mb;
int16_t mc;
int16_t md;

int16_t temperature;
int32_t pressure;
int16_t altitude;

//get data
uint16_t BMP085_Read2(unsigned char address)
	{
	uint8_t response[2];
	
	if(!i2cMasterReadRegisters(&Pressure_I2C.i2cInfo, address, sizeof(response), response))
		rprintf("\npressure read failed");

	//combine values
	//rprintf("\nresponse0:%u response1:%u", response[0],response[1]);
	return ((response[0] << 8) | response[1]);
	}

//calibrate the sensor
void BMP085_Calibration(void)
	{
	ac1 = BMP085_Read2(0xAA);
	ac2 = BMP085_Read2(0xAC);
	ac3 = BMP085_Read2(0xAE);
	ac4 = BMP085_Read2(0xB0);
	ac5 = BMP085_Read2(0xB2);
	ac6 = BMP085_Read2(0xB4);
	b1 = BMP085_Read2(0xB6);
	b2 = BMP085_Read2(0xB8);
	mb = BMP085_Read2(0xBA);
	mc = BMP085_Read2(0xBC);
	md = BMP085_Read2(0xBE);

	/*rprintf("\n\nPressure Sensor Calibration Coefficients");
	rprintf("\nAC1 = %d", ac1);
	rprintf("\nAC2 = %d", ac2);
	rprintf("\nAC3 = %d", ac3);
	rprintf("\nAC4 = %u", ac4);
	rprintf("\nAC5 = %u", ac5);
	rprintf("\nAC6 = %u", ac6);
	rprintf("\nB1 = %d", b1);
	rprintf("\nB2 = %d", b2);
	rprintf("\nMB = %d", mb);
	rprintf("\nMC = %d", mc);
	rprintf("\nMD = %d\n", md);*/
	}

//read temperature
int32_t BMP085_ReadTemp(void)
	{
	//ask for temp
	i2cMasterWriteRegister(&Pressure_I2C.i2cInfo, 0xF4, 0x2E);

	delay_ms(7);	// max time is 4.5ms
	
	//receive temp, UT = MSB<<8 + LSB;
	
	return BMP085_Read2(0xF6);
	}

//read pressure
int32_t BMP085_ReadPres(void)
	{
	int32_t pres;
	uint8_t response[3];

	//ask for pres
	// OSS    Reg 0xF4
	//   0	    0x34
	//   1      0x74
	//   2      0xB4
	//   3	    0xF4
	i2cMasterWriteRegister(&Pressure_I2C.i2cInfo, 0xF4, (0x34 + (OSS<<6)));

	delay_ms(26);	// max time is 26ms for OSS 3
	
	
	if(!i2cMasterReadRegisters(&Pressure_I2C.i2cInfo, 0xF6, sizeof(response), response))
		rprintf("\npressure read failed");


	//receive pres, UP = (MSB<<16 + LSB<<8 + XLSB) >> (8-oss)
	pres = response[0];
	pres<<=8; pres |= response[1];
	pres<<=8; pres |= response[2];
	pres>>=(8-OSS);

	return pres;
	}

//get temperature pressure and altitude, returns pressure in Pa, temp in C, altitude in m
void BMP085_Convert(void)
	{
	int32_t ut;
	int32_t up;
	int32_t x1, x2, b5, b6, x3, b3, p;
	uint32_t b4, b7;
	
	ut = BMP085_ReadTemp();
	up = BMP085_ReadPres();//washington dc should be 101.33kpa
	//rprintf("\nut=%ld",ut);
	//rprintf("\nup=%ld",up);
	
	//TEST - force values to match datasheet example
	/*ut = 27898;
	up = 23843;
	ac1 = 408;
	ac2 = -72;
	ac3 = -14383;
	ac4 = 32741;
	ac5 = 32757;
	ac6 = 23153;
	b1 = 6190;
	b2 = 4;
	mb = -32768;
	mc = -8711;
	md = 2868;*/
	
	//calculate true temperature
	x1 = ( ut - ac6) * ac5 >> 15;
	x2 = ( (int32_t)mc << 11) / (x1 + md);
	b5 = x1 + x2;
	temperature = (b5 + 8) >> 4;

	/*rprintf("\nx1=%ld ",x1);
	rprintf("x2=%ld ",x2);
	rprintf("b5=%ld",b5);*/
	
	//calculate true pressure
	b6 = b5 - 4000;
//	rprintf("\nb6 %ld",b6);
	x1 = (b2 * (b6 * b6 >> 12)) >> 11;
//	rprintf("\nx1 %ld",x1);
	x2 = ac2 * b6 >> 11;
//	rprintf("\nx2 %ld",x2);
	x3 = x1 + x2;
//	rprintf("\nx3 %ld",x3);
	b3 = (( ((int32_t)ac1 * 4 + x3) << OSS ) + 2)/4;
//	rprintf("\nb3 %ld",b3);
	x1 = ac3 * b6 >> 13;
//	rprintf("\nx1 %ld",x1);
	x2 = (b1 * (b6 * b6 >> 12)) >> 16;
//	rprintf("\nx2 %ld",x2);
	x3 = ((x1 + x2) + 2) >> 2;
//	rprintf("\nx3 %ld",x3);
	b4 = ac4 * (uint32_t)(x3 + 32768) >> 15;
//	rprintf("\nb4 %lu",b4);
	b7 = ( (uint32_t)up - b3) * (50000 >> OSS);
//	rprintf("\nb7 %lu",b7);
	p = b7 < 0x80000000 ? (b7 * 2) / b4 : (b7 / b4) * 2;
//	rprintf("\np %ld",p);
	x1 = (p >> 8) * (p >> 8);
//	rprintf("\nx1 %ld",x1);
	x1 = (x1 * 3038) >> 16;
//	rprintf("\nx1 %ld",x1);
	x2 = (-7357 * p) >> 16;
//	rprintf("\nx2 %ld",x2);

	pressure = p + ((x1 + x2 + 3791) >> 4);

	//convert to altitude
	altitude = 44330.75*(1-pow((double)pressure/101325, 0.19029))+tweak_altitude;
	}
